@@ -1,15 +1,15 @@
-//! [`HashMap`] implementation.
+//! The module implements [`HashMap`].
 
-use crate::common::cell::Locker;
-use crate::common::cell_array::CellArray;
-use crate::common::hash_table::HashTable;
-use crate::ebr::{Arc, AtomicArc, Barrier, Tag};
+use super::ebr::{Arc, AtomicArc, Barrier, Tag};
+use super::hash_table::cell::Locker;
+use super::hash_table::cell_array::CellArray;
+use super::hash_table::HashTable;
 
 use std::borrow::Borrow;
 use std::collections::hash_map::RandomState;
 use std::hash::{BuildHasher, Hash};
 use std::sync::atomic::Ordering::{Acquire, Relaxed};
-use std::sync::atomic::{AtomicBool, AtomicUsize};
+use std::sync::atomic::{AtomicU8, AtomicUsize};
 
 const CELL_SIZE: usize = 32;
 const DEFAULT_CAPACITY: usize = 64;
@@ -19,26 +19,27 @@ const DEFAULT_CAPACITY: usize = 64;
 /// [`HashMap`] is a concurrent hash map data structure that is targeted at a highly concurrent
 /// workload. The use of an epoch-based reclamation technique enables the data structure to
 /// implement non-blocking resizing and fine-granular locking. It has a single array of entry
-/// metadata, and each entry, called `Cell`, manages a fixed size entry array. Each `Cell` has
-/// a customized 8-byte read-write mutex to protect the data structure, and a linked list for a
-/// hash collision resolution.
+/// metadata, and each entry of the array, called `Cell`, manages a fixed size key-value pair
+/// array. Each `Cell` has a customized 8-byte read-write mutex to protect the data structure,
+/// and a linked list for resolving hash collisions.
 ///
 /// ## The key features of [`HashMap`]
+///
 /// * Non-sharded: the data is managed by a single entry metadata array.
 /// * Automatic resizing: it automatically grows or shrinks.
 /// * Non-blocking resizing: resizing does not block other threads.
 /// * Incremental resizing: each access to the data structure is mandated to rehash a fixed
 ///   number of key-value pairs.
-/// * Optimized resizing: key-value pairs managed by a single cell are guaranteed to be
+/// * Optimized resizing: key-value pairs managed by a single `Cell` are guaranteed to be
 ///   relocated to consecutive `Cell` instances.
 /// * No busy waiting: the customized mutex never spins.
 ///
 /// ## The key statistics for [`HashMap`]
+///
 /// * The expected size of metadata for a single key-value pair: 2-byte.
 /// * The expected number of atomic operations required for an operation on a single key: 2.
 /// * The expected number of atomic variables accessed during a single key operation: 1.
 /// * The number of entries managed by a single metadata `Cell` without a linked list: 32.
-/// * The number of entries a single linked list entry manages: 8.
 /// * The expected maximum linked list length when resize is triggered: log(capacity) / 8.
 pub struct HashMap<K, V, H = RandomState>
 where
@@ -49,47 +50,10 @@ where
     array: AtomicArc<CellArray<K, V, CELL_SIZE, false>>,
     minimum_capacity: usize,
     additional_capacity: AtomicUsize,
-    resizing_flag: AtomicBool,
+    resize_mutex: AtomicU8,
     build_hasher: H,
 }
 
-impl<K, V> Default for HashMap<K, V, RandomState>
-where
-    K: 'static + Eq + Hash + Sync,
-    V: 'static + Sync,
-{
-    /// Creates a [`HashMap`] with the default parameters.
-    ///
-    /// The default hash builder is [`RandomState`], and the default capacity is `64`.
-    ///
-    /// # Panics
-    ///
-    /// Panics if memory allocation fails.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::HashMap;
-    ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
-    ///
-    /// let result = hashmap.capacity();
-    /// assert_eq!(result, 64);
-    /// ```
-    fn default() -> Self {
-        HashMap {
-            array: AtomicArc::new(CellArray::<K, V, CELL_SIZE, false>::new(
-                DEFAULT_CAPACITY,
-                AtomicArc::null(),
-            )),
-            minimum_capacity: DEFAULT_CAPACITY,
-            additional_capacity: AtomicUsize::new(0),
-            resizing_flag: AtomicBool::new(false),
-            build_hasher: RandomState::new(),
-        }
-    }
-}
-
 impl<K, V, H> HashMap<K, V, H>
 where
     K: 'static + Eq + Hash + Sync,
@@ -115,7 +79,7 @@ where
     /// let result = hashmap.capacity();
     /// assert_eq!(result, 1024);
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     /// let result = hashmap.capacity();
     /// assert_eq!(result, 64);
     /// ```
@@ -125,12 +89,12 @@ where
             initial_capacity,
             AtomicArc::null(),
         ));
-        let current_capacity = array.num_cell_entries();
+        let current_capacity = array.num_entries();
         HashMap {
             array: AtomicArc::from(array),
             minimum_capacity: current_capacity,
             additional_capacity: AtomicUsize::new(0),
-            resizing_flag: AtomicBool::new(false),
+            resize_mutex: AtomicU8::new(0),
             build_hasher,
         }
     }
@@ -142,7 +106,7 @@ where
     ///
     /// # Errors
     ///
-    /// Returns [`None`] if a too large number is given.
+    /// Returns `None` if a too large number is given.
     ///
     /// # Examples
     ///
@@ -187,6 +151,7 @@ where
             }
         }
     }
+
     /// Inserts a key-value pair into the [`HashMap`].
     ///
     /// # Errors
@@ -195,15 +160,15 @@ where
     ///
     /// # Panics
     ///
-    /// Panics if memory allocation fails, or the number of entries in the target cell is
-    /// reached `u32::MAX`.
+    /// Panics if memory allocation fails, or the number of entries in the target cell reaches
+    /// `u32::MAX`.
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert_eq!(hashmap.insert(1, 1).unwrap_err(), (1, 1));
@@ -215,14 +180,14 @@ where
 
     /// Updates an existing key-value pair.
     ///
-    /// Returns `None` if the key does not exist.
+    /// It returns `None` if the key does not exist.
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.update(&1, |_, _| true).is_none());
     /// assert!(hashmap.insert(1, 0).is_ok());
@@ -261,7 +226,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// hashmap.upsert(1, || 2, |_, v| *v = 2);
     /// assert_eq!(hashmap.read(&1, |_, v| *v).unwrap(), 2);
@@ -289,14 +254,14 @@ where
         locker.insert(key, constructor(), partial_hash, &barrier);
     }
 
-    /// Removes a key-value pair and returns the key-value-pair if the key exists.
+    /// Removes a key-value pair if the key exists.
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.remove(&1).is_none());
     /// assert!(hashmap.insert(1, 0).is_ok());
@@ -311,15 +276,14 @@ where
         self.remove_if(key_ref, |_| true)
     }
 
-    /// Removes a key-value pair and returns the key-value-pair if the key exists and the given
-    /// condition meets.
+    /// Removes a key-value pair if the key exists and the given condition is met.
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert!(hashmap.remove_if(&1, |v| *v == 1).is_none());
@@ -331,55 +295,23 @@ where
         K: Borrow<Q>,
         Q: Eq + Hash + ?Sized,
     {
-        let (hash, partial_hash) = self.hash(key_ref);
-        let barrier = Barrier::new();
-        let (cell_index, locker, iterator) = self.acquire(key_ref, hash, partial_hash, &barrier);
-        if let Some(mut iterator) = iterator {
-            let remove = if let Some((_, v)) = iterator.get() {
-                condition(v)
-            } else {
-                false
-            };
-            if remove {
-                let result = locker.erase(&mut iterator);
-                if cell_index < CELL_SIZE && locker.cell_ref().num_entries() < CELL_SIZE / 16 {
-                    drop(locker);
-                    let current_array_ptr = self.array.load(Acquire, &barrier);
-                    if let Some(current_array_ref) = current_array_ptr.as_ref() {
-                        if current_array_ref.old_array(&barrier).is_null()
-                            && current_array_ref.num_cell_entries() > self.minimum_capacity()
-                        {
-                            let sample_size = current_array_ref.sample_size();
-                            let mut num_entries = 0;
-                            for i in 0..sample_size {
-                                num_entries += current_array_ref.cell(i).num_entries();
-                                if num_entries >= sample_size * CELL_SIZE / 16 {
-                                    return result;
-                                }
-                            }
-                            self.resize(&barrier);
-                        }
-                    }
-                }
-                return result;
-            }
-        }
-        None
+        self.remove_entry(key_ref, condition).0
     }
 
     /// Reads a key-value pair.
     ///
-    /// Returns `None` if the key does not exist.
+    /// It returns `None` if the key does not exist.
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
-    /// assert!(hashmap.insert(1, 0).is_ok());
-    /// assert_eq!(hashmap.read(&1, |_, v| *v).unwrap(), 0);
+    /// assert!(hashmap.read(&1, |_, v| *v).is_none());
+    /// assert!(hashmap.insert(1, 10).is_ok());
+    /// assert_eq!(hashmap.read(&1, |_, v| *v).unwrap(), 10);
     /// ```
     #[inline]
     pub fn read<Q, R, F: FnOnce(&K, &V) -> R>(&self, key_ref: &Q, reader: F) -> Option<R>
@@ -387,7 +319,41 @@ where
         K: Borrow<Q>,
         Q: Eq + Hash + ?Sized,
     {
-        self.read_entry(key_ref, reader)
+        let barrier = Barrier::new();
+        self.read_with(key_ref, reader, &barrier)
+    }
+
+    /// Reads a key-value pair using the supplied [`Barrier`].
+    ///
+    /// It enables the caller to use the value reference outside the method. It returns `None`
+    /// if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::ebr::Barrier;
+    /// use scc::HashMap;
+    ///
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
+    ///
+    /// assert!(hashmap.insert(1, 10).is_ok());
+    ///
+    /// let barrier = Barrier::new();
+    /// let value_ref = hashmap.read_with(&1, |k, v| v, &barrier).unwrap();
+    /// assert_eq!(*value_ref, 10);
+    /// ```
+    #[inline]
+    pub fn read_with<'b, Q, R, F: FnOnce(&'b K, &'b V) -> R>(
+        &self,
+        key_ref: &Q,
+        reader: F,
+        barrier: &'b Barrier,
+    ) -> Option<R>
+    where
+        K: Borrow<Q>,
+        Q: Eq + Hash + ?Sized,
+    {
+        self.read_entry(key_ref, reader, barrier)
     }
 
     /// Checks if the key exists.
@@ -397,7 +363,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(!hashmap.contains(&1));
     /// assert!(hashmap.insert(1, 0).is_ok());
@@ -412,14 +378,14 @@ where
         self.read(key, |_, _| ()).is_some()
     }
 
-    /// It iterates over all the entries in the [`HashMap`].
+    /// Iterates over all the entries in the [`HashMap`].
     ///
     /// # Examples
     ///
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert!(hashmap.insert(2, 1).is_ok());
@@ -438,7 +404,7 @@ where
         });
     }
 
-    /// Retains the key-value pairs that satisfy the given predicate.
+    /// Retains key-value pairs that satisfy the given predicate.
     ///
     /// It returns the number of entries remaining and removed.
     ///
@@ -447,7 +413,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert!(hashmap.insert(2, 1).is_ok());
@@ -470,7 +436,7 @@ where
                 continue;
             }
 
-            for cell_index in 0..current_array_ref.array_size() {
+            for cell_index in 0..current_array_ref.num_cells() {
                 if let Some(locker) = Locker::lock(current_array_ref.cell(cell_index), &barrier) {
                     let mut iterator = locker.cell_ref().iter(&barrier);
                     while iterator.next().is_some() {
@@ -516,7 +482,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert_eq!(hashmap.clear(), 1);
@@ -536,7 +502,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.insert(1, 0).is_ok());
     /// assert_eq!(hashmap.len(), 1);
@@ -556,7 +522,7 @@ where
     /// ```
     /// use scc::HashMap;
     ///
-    /// let hashmap: HashMap<u64, u32> = Default::default();
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
     /// assert!(hashmap.is_empty());
     /// ```
@@ -582,6 +548,43 @@ where
     }
 }
 
+impl<K, V> Default for HashMap<K, V, RandomState>
+where
+    K: 'static + Eq + Hash + Sync,
+    V: 'static + Sync,
+{
+    /// Creates a [`HashMap`] with the default parameters.
+    ///
+    /// The default hash builder is [`RandomState`], and the default capacity is `64`.
+    ///
+    /// # Panics
+    ///
+    /// Panics if memory allocation fails.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashMap;
+    ///
+    /// let hashmap: HashMap<u64, u32> = HashMap::default();
+    ///
+    /// let result = hashmap.capacity();
+    /// assert_eq!(result, 64);
+    /// ```
+    fn default() -> Self {
+        HashMap {
+            array: AtomicArc::new(CellArray::<K, V, CELL_SIZE, false>::new(
+                DEFAULT_CAPACITY,
+                AtomicArc::null(),
+            )),
+            minimum_capacity: DEFAULT_CAPACITY,
+            additional_capacity: AtomicUsize::new(0),
+            resize_mutex: AtomicU8::new(0),
+            build_hasher: RandomState::new(),
+        }
+    }
+}
+
 impl<K, V, H> Drop for HashMap<K, V, H>
 where
     K: 'static + Eq + Hash + Sync,
@@ -619,15 +622,15 @@ where
     fn minimum_capacity(&self) -> usize {
         self.minimum_capacity + self.additional_capacity.load(Relaxed)
     }
-    fn resizing_flag_ref(&self) -> &AtomicBool {
-        &self.resizing_flag
+    fn resize_mutex(&self) -> &AtomicU8 {
+        &self.resize_mutex
     }
 }
 
 /// [`Ticket`] keeps the increased minimum capacity of the [`HashMap`] during its lifetime.
 ///
-/// The minimum capacity is lowered when the Ticket is dropped, thereby allowing unused space
-/// to be reclaimed.
+/// The minimum capacity is lowered when the [`Ticket`] is dropped, thereby allowing unused
+/// memory to be reclaimed.
 pub struct Ticket<'h, K, V, H>
 where
     K: 'static + Eq + Hash + Sync,
