@@ -32,9 +32,12 @@ pub use hash_map::HashMap;
 pub mod hash_index;
 pub use hash_index::HashIndex;
 
+pub mod hash_set;
+pub use hash_set::HashSet;
+
 pub mod tree_index;
 pub use tree_index::TreeIndex;
 
-mod common;
+mod hash_table;
 
 mod tests;
