@@ -1,4 +1,4 @@
-//! [`TreeIndex`] implementation.
+//! The module implements [`TreeIndex`].
 
 mod error;
 mod leaf;
@@ -25,10 +25,12 @@ use std::sync::atomic::Ordering::{AcqRel, Acquire, Relaxed};
 /// such as insert, remove, do not block if they do not entail structural changes to the tree.
 ///
 /// ## The key features of [`TreeIndex`]
+///
 /// * Write-free read: read operations never modify the shared data.
 /// * Near lock-free write: write operations do not block unless a structural change is needed.
 ///
 /// ## The key statistics for [`TreeIndex`]
+///
 /// * The maximum number of key-value pairs that a leaf can store: 8.
 /// * The maximum number of leaves or child nodes that a node can point to: 9.
 /// * The size of metadata per key-value pair in a leaf: 3-byte.
@@ -41,27 +43,6 @@ where
     root: AtomicArc<Node<K, V>>,
 }
 
-impl<K, V> Default for TreeIndex<K, V>
-where
-    K: 'static + Clone + Ord + Send + Sync,
-    V: 'static + Clone + Send + Sync,
-{
-    /// Creates a [`TreeIndex`] with the default parameters.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::TreeIndex;
-    ///
-    /// let treeindex: TreeIndex<u64, u32> = Default::default();
-    ///
-    /// assert!(treeindex.read(&1, |_, v| *v).is_none());
-    /// ```
-    fn default() -> Self {
-        TreeIndex::new()
-    }
-}
-
 impl<K, V> TreeIndex<K, V>
 where
     K: 'static + Clone + Ord + Send + Sync,
@@ -208,6 +189,8 @@ where
 
     /// Reads a key-value pair.
     ///
+    /// It returns `None` if the key does not exist.
+    ///
     /// # Examples
     ///
     /// ```
@@ -220,18 +203,51 @@ where
     /// assert_eq!(treeindex.read(&1, |k, v| *v).unwrap(), 10);
     /// ```
     #[inline]
-    pub fn read<Q, R, F: FnOnce(&Q, &V) -> R>(&self, key_ref: &Q, f: F) -> Option<R>
+    pub fn read<Q, R, F: FnOnce(&Q, &V) -> R>(&self, key_ref: &Q, reader: F) -> Option<R>
     where
         K: Borrow<Q>,
         Q: Ord + ?Sized,
     {
         let barrier = Barrier::new();
-        let mut root_ptr = self.root.load(Acquire, &barrier);
+        self.read_with(key_ref, reader, &barrier)
+    }
+
+    /// Reads a key-value pair using the supplied [`Barrier`].
+    ///
+    /// It enables the caller to use the value reference outside the method. It returns `None`
+    /// if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::ebr::Barrier;
+    /// use scc::TreeIndex;
+    ///
+    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
+    ///
+    /// assert!(treeindex.insert(1, 10).is_ok());
+    ///
+    /// let barrier = Barrier::new();
+    /// let value_ref = treeindex.read_with(&1, |k, v| v, &barrier).unwrap();
+    /// assert_eq!(*value_ref, 10);
+    /// ```
+    #[inline]
+    pub fn read_with<'b, Q, R, F: FnOnce(&Q, &'b V) -> R>(
+        &self,
+        key_ref: &Q,
+        reader: F,
+        barrier: &'b Barrier,
+    ) -> Option<R>
+    where
+        K: Borrow<Q>,
+        Q: Ord + ?Sized,
+    {
+        let mut root_ptr = self.root.load(Acquire, barrier);
         while let Some(root_ref) = root_ptr.as_ref() {
-            match root_ref.search(key_ref, &barrier) {
+            match root_ref.search(key_ref, barrier) {
                 Ok(result) => {
                     if let Some(value) = result {
-                        return Some(f(key_ref, value));
+                        return Some(reader(key_ref, value));
                     }
                     return None;
                 }
@@ -239,7 +255,7 @@ where
                     SearchError::Empty => return None,
                     SearchError::Retry => {
                         std::thread::yield_now();
-                        root_ptr = self.root.load(Acquire, &barrier);
+                        root_ptr = self.root.load(Acquire, barrier);
                         continue;
                     }
                 },
@@ -427,6 +443,27 @@ where
     }
 }
 
+impl<K, V> Default for TreeIndex<K, V>
+where
+    K: 'static + Clone + Ord + Send + Sync,
+    V: 'static + Clone + Send + Sync,
+{
+    /// Creates a [`TreeIndex`] with the default parameters.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::TreeIndex;
+    ///
+    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::default();
+    ///
+    /// assert!(treeindex.read(&1, |_, v| *v).is_none());
+    /// ```
+    fn default() -> Self {
+        TreeIndex::new()
+    }
+}
+
 /// [`Visitor`] scans all the key-value pairs in the [`TreeIndex`].
 ///
 /// It is guaranteed to visit all the key-value pairs that outlive the [`Visitor`], and it
